/*     
 *  This file is part of DEW browser.
 *  Copyright (C) 2023 Woohyun Joh 
 *
 *  DEW browesr is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as
 *  published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  DEW browesr is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

pub mod content;
mod iii;
mod mmm;

use crate::iii::InteractiveImage;
use crate::mmm::MarkdownMarkupModel;

use std::io::{Error, ErrorKind};

pub enum Document {
    MMM(MarkdownMarkupModel),
    III(InteractiveImage),
}

impl TryFrom<&[u8]> for Document {
    type Error = Error;
    fn try_from(slice: &[u8]) -> Result<Document, Error> {
        let document_type = slice[0];
        let document_slice = &slice[1..];
        match document_type {
            b'M' => Ok(Document::MMM(MarkdownMarkupModel::try_from(
                document_slice,
            )?)),
            b'I' => Ok(Document::III(InteractiveImage::from(document_slice))),
            _ => Err(Error::from(ErrorKind::InvalidInput)),
        }
    }
}
