/*     
 *  This file is part of DEW browser.
 *  Copyright (C) 2023 Woohyun Joh 
 *
 *  DEW browesr is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as
 *  published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  DEW browesr is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

use crate::ScriptError;

#[derive(Default, PartialEq, Clone)]
pub struct Array<T> {
    vec: Vec<T>,
}

impl<T> From<Vec<T>> for Array<T> {
    fn from(vec: Vec<T>) -> Array<T> {
        Array { vec }
    }
}

impl<T> Array<T> {
    #[inline]
    pub fn push(&mut self, new: T) {
        self.vec.push(new);
    }

    #[inline]
    pub fn index(&self, idx: usize) -> Result<&T, ScriptError> {
        if idx >= self.vec.len() {
            Err(ScriptError::OutOfArray)
        } else {
            Ok(&self.vec[idx])
        }
    }

    #[inline]
    pub fn index_mut(&mut self, idx: usize) -> Result<&mut T, ScriptError> {
        if idx >= self.vec.len() {
            Err(ScriptError::OutOfArray)
        } else {
            Ok(&mut self.vec[idx])
        }
    }

    #[inline]
    pub fn truncate(&mut self, at: usize) {
        self.vec.truncate(at)
    }

    #[inline]
    pub fn len(&self) -> usize {
        self.vec.len()
    }
}
