/*     
 *  This file is part of DEW browser.
 *  Copyright (C) 2023 Woohyun Joh 
 *
 *  DEW browesr is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as
 *  published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  DEW browesr is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

use crate::ScriptError;

#[derive(Default)]
pub struct Stack<T> {
    vec: Vec<T>,
}

impl<T> Stack<T> {
    #[inline]
    pub fn push(&mut self, item: T) {
        self.vec.push(item);
    }

    pub fn pop(&mut self) -> Result<T, ScriptError> {
        match self.vec.pop() {
            Some(item) => Ok(item),
            None => Err(ScriptError::EmptyStack),
        }
    }

    pub fn last(&self) -> Result<&T, ScriptError> {
        match self.vec.last() {
            Some(item) => Ok(item),
            None => Err(ScriptError::EmptyStack),
        }
    }

    #[inline]
    pub fn is_empty(&self) -> bool {
        self.vec.is_empty()
    }

    #[inline]
    pub fn truncate(&mut self, at: usize) {
        self.vec.truncate(at);
    }

    #[inline]
    pub fn len(&self) -> usize {
        self.vec.len()
    }
}
